@@ -1,13 +1,12 @@
 """
 Meta Ads MCP - Python Package
 
-This package provides a Meta Ads Marketing Cloud Platform (MCP) integration
-with the Claude LLM.
+This package provides a Meta Ads MCP integration
 """
 
 from meta_ads_mcp.core.server import main
 
-__version__ = "0.10.5"
+__version__ = "1.0.2"
 
 __all__ = [
     'get_ad_accounts',
@@ -24,7 +23,7 @@ __all__ = [
     'get_ad_image',
     'update_ad',
     'get_insights',
-    'get_login_link',
+    # 'get_login_link' is conditionally exported via core.__all__
     'login_cli',
     'main',
     'search_interests',
@@ -51,7 +50,6 @@ from .core import (
     get_ad_image,
     update_ad,
     get_insights,
-    get_login_link,
     login_cli,
     main,
     search_interests,
