@@ -9,38 +9,32 @@ from .server import mcp_server
 
 @mcp_server.tool()
 @meta_api_tool
-async def get_adsets(access_token: str = None, account_id: str = None, limit: int = 10, campaign_id: str = "") -> str:
+async def get_adsets(account_id: str, access_token: Optional[str] = None, limit: int = 10, campaign_id: str = "") -> str:
     """
     Get ad sets for a Meta Ads account with optional filtering by campaign.
     
     Args:
-        access_token: Meta API access token (optional - will use cached token if not provided)
         account_id: Meta Ads account ID (format: act_XXXXXXXXX)
+        access_token: Meta API access token (optional - will use cached token if not provided)
         limit: Maximum number of ad sets to return (default: 10)
         campaign_id: Optional campaign ID to filter by
     """
-    # If no account ID is specified, try to get the first one for the user
+    # Require explicit account_id
     if not account_id:
-        accounts_json = await get_ad_accounts("me", json.dumps({"limit": 1}), access_token)
-        accounts_data = json.loads(accounts_json)
-        
-        if "data" in accounts_data and accounts_data["data"]:
-            account_id = accounts_data["data"][0]["id"]
-        else:
-            return json.dumps({"error": "No account ID specified and no accounts found for user"}, indent=2)
+        return json.dumps({"error": "No account ID specified"}, indent=2)
     
     # Change endpoint based on whether campaign_id is provided
     if campaign_id:
         endpoint = f"{campaign_id}/adsets"
         params = {
-            "fields": "id,name,campaign_id,status,daily_budget,lifetime_budget,targeting,bid_amount,bid_strategy,optimization_goal,billing_event,start_time,end_time,created_time,updated_time,frequency_control_specs{event,interval_days,max_frequency}",
+            "fields": "id,name,campaign_id,status,daily_budget,lifetime_budget,targeting,bid_amount,bid_strategy,optimization_goal,billing_event,start_time,end_time,created_time,updated_time,is_dynamic_creative,frequency_control_specs{event,interval_days,max_frequency}",
             "limit": limit
         }
     else:
         # Use account endpoint if no campaign_id is given
         endpoint = f"{account_id}/adsets"
         params = {
-            "fields": "id,name,campaign_id,status,daily_budget,lifetime_budget,targeting,bid_amount,bid_strategy,optimization_goal,billing_event,start_time,end_time,created_time,updated_time,frequency_control_specs{event,interval_days,max_frequency}",
+            "fields": "id,name,campaign_id,status,daily_budget,lifetime_budget,targeting,bid_amount,bid_strategy,optimization_goal,billing_event,start_time,end_time,created_time,updated_time,is_dynamic_creative,frequency_control_specs{event,interval_days,max_frequency}",
             "limit": limit
         }
         # Note: Removed the attempt to add campaign_id to params for the account endpoint case, 
@@ -53,12 +47,12 @@ async def get_adsets(access_token: str = None, account_id: str = None, limit: in
 
 @mcp_server.tool()
 @meta_api_tool
-async def get_adset_details(access_token: str = None, adset_id: str = None) -> str:
+async def get_adset_details(adset_id: str, access_token: Optional[str] = None) -> str:
     """
     Get detailed information about a specific ad set.
     
     Args:
-        adset_id: Meta Ads ad set ID (required)
+        adset_id: Meta Ads ad set ID
         access_token: Meta API access token (optional - will use cached token if not provided)
     
     Example:
@@ -73,7 +67,7 @@ async def get_adset_details(access_token: str = None, adset_id: str = None) -> s
     endpoint = f"{adset_id}"
     # Explicitly prioritize frequency_control_specs in the fields request
     params = {
-        "fields": "id,name,campaign_id,status,frequency_control_specs{event,interval_days,max_frequency},daily_budget,lifetime_budget,targeting,bid_amount,bid_strategy,optimization_goal,billing_event,start_time,end_time,created_time,updated_time,attribution_spec,destination_type,promoted_object,pacing_type,budget_remaining,dsa_beneficiary"
+        "fields": "id,name,campaign_id,status,frequency_control_specs{event,interval_days,max_frequency},daily_budget,lifetime_budget,targeting,bid_amount,bid_strategy,optimization_goal,billing_event,start_time,end_time,created_time,updated_time,attribution_spec,destination_type,promoted_object,pacing_type,budget_remaining,dsa_beneficiary,is_dynamic_creative"
     }
     
     data = await make_api_request(endpoint, access_token, params)
@@ -90,23 +84,24 @@ async def get_adset_details(access_token: str = None, adset_id: str = None) -> s
 @mcp_server.tool()
 @meta_api_tool
 async def create_adset(
-    account_id: str = None, 
-    campaign_id: str = None, 
-    name: str = None,
+    account_id: str, 
+    campaign_id: str, 
+    name: str,
+    optimization_goal: str,
+    billing_event: str,
     status: str = "PAUSED",
-    daily_budget = None,
-    lifetime_budget = None,
-    targeting: Dict[str, Any] = None,
-    optimization_goal: str = None,
-    billing_event: str = None,
-    bid_amount = None,
-    bid_strategy: str = None,
-    start_time: str = None,
-    end_time: str = None,
-    dsa_beneficiary: str = None,
-    promoted_object: Dict[str, Any] = None,
-    destination_type: str = None,
-    access_token: str = None
+    daily_budget: Optional[int] = None,
+    lifetime_budget: Optional[int] = None,
+    targeting: Optional[Dict[str, Any]] = None,
+    bid_amount: Optional[int] = None,
+    bid_strategy: Optional[str] = None,
+    start_time: Optional[str] = None,
+    end_time: Optional[str] = None,
+    dsa_beneficiary: Optional[str] = None,
+    promoted_object: Optional[Dict[str, Any]] = None,
+    destination_type: Optional[str] = None,
+    is_dynamic_creative: Optional[bool] = None,
+    access_token: Optional[str] = None
 ) -> str:
     """
     Create a new ad set in a Meta Ads account.
@@ -115,13 +110,13 @@ async def create_adset(
         account_id: Meta Ads account ID (format: act_XXXXXXXXX)
         campaign_id: Meta Ads campaign ID this ad set belongs to
         name: Ad set name
+        optimization_goal: Conversion optimization goal (e.g., 'LINK_CLICKS', 'REACH', 'CONVERSIONS', 'APP_INSTALLS')
+        billing_event: How you're charged (e.g., 'IMPRESSIONS', 'LINK_CLICKS')
         status: Initial ad set status (default: PAUSED)
         daily_budget: Daily budget in account currency (in cents) as a string
         lifetime_budget: Lifetime budget in account currency (in cents) as a string
         targeting: Targeting specifications including age, location, interests, etc.
                   Use targeting_automation.advantage_audience=1 for automatic audience finding
-        optimization_goal: Conversion optimization goal (e.g., 'LINK_CLICKS', 'REACH', 'CONVERSIONS', 'APP_INSTALLS')
-        billing_event: How you're charged (e.g., 'IMPRESSIONS', 'LINK_CLICKS')
         bid_amount: Bid amount in account currency (in cents)
         bid_strategy: Bid strategy (e.g., 'LOWEST_COST', 'LOWEST_COST_WITH_BID_CAP')
         start_time: Start time in ISO 8601 format (e.g., '2023-12-01T12:00:00-0800')
@@ -131,8 +126,9 @@ async def create_adset(
                         Optional fields: custom_event_type, pixel_id, page_id.
                         Example: {"application_id": "123456789012345", "object_store_url": "https://apps.apple.com/app/id123456789"}
         destination_type: Where users are directed after clicking the ad (e.g., 'APP_STORE', 'DEEPLINK', 'APP_INSTALL', 'ON_AD').
-                         Required for mobile app campaigns and lead generation campaigns.
-                         Use 'ON_AD' for lead generation campaigns where user interaction happens within the ad.
+                          Required for mobile app campaigns and lead generation campaigns.
+                          Use 'ON_AD' for lead generation campaigns where user interaction happens within the ad.
+        is_dynamic_creative: Enable Dynamic Creative for this ad set (required when using dynamic creatives with asset_feed_spec/dynamic_creative_spec).
         access_token: Meta API access token (optional - will use cached token if not provided)
     """
     # Check required parameters
@@ -255,6 +251,10 @@ async def create_adset(
     if destination_type:
         params["destination_type"] = destination_type
     
+    # Enable Dynamic Creative if requested
+    if is_dynamic_creative is not None:
+        params["is_dynamic_creative"] = "true" if bool(is_dynamic_creative) else "false"
+    
     try:
         data = await make_api_request(endpoint, access_token, params, method="POST")
         return json.dumps(data, indent=2)
@@ -293,10 +293,11 @@ async def create_adset(
 
 @mcp_server.tool()
 @meta_api_tool
-async def update_adset(adset_id: str, frequency_control_specs: List[Dict[str, Any]] = None, bid_strategy: str = None, 
-                        bid_amount: int = None, status: str = None, targeting: Dict[str, Any] = None, 
-                        optimization_goal: str = None, daily_budget = None, lifetime_budget = None, 
-                        access_token: str = None) -> str:
+async def update_adset(adset_id: str, frequency_control_specs: Optional[List[Dict[str, Any]]] = None, bid_strategy: Optional[str] = None, 
+                        bid_amount: Optional[int] = None, status: Optional[str] = None, targeting: Optional[Dict[str, Any]] = None, 
+                        optimization_goal: Optional[str] = None, daily_budget: Optional[int] = None, lifetime_budget: Optional[int] = None, 
+                        is_dynamic_creative: Optional[bool] = None,
+                        access_token: Optional[str] = None) -> str:
     """
     Update an ad set with new settings including frequency caps and budgets.
     
@@ -312,6 +313,7 @@ async def update_adset(adset_id: str, frequency_control_specs: List[Dict[str, An
         optimization_goal: Conversion optimization goal (e.g., 'LINK_CLICKS', 'CONVERSIONS', 'APP_INSTALLS', etc.)
         daily_budget: Daily budget in account currency (in cents) as a string
         lifetime_budget: Lifetime budget in account currency (in cents) as a string
+        is_dynamic_creative: Enable/disable Dynamic Creative for this ad set.
         access_token: Meta API access token (optional - will use cached token if not provided)
     """
     if not adset_id:
@@ -348,6 +350,9 @@ async def update_adset(adset_id: str, frequency_control_specs: List[Dict[str, An
     if lifetime_budget is not None:
         params['lifetime_budget'] = str(lifetime_budget)
     
+    if is_dynamic_creative is not None:
+        params['is_dynamic_creative'] = "true" if bool(is_dynamic_creative) else "false"
+    
     if not params:
         return json.dumps({"error": "No update parameters provided"}, indent=2)
 
