@@ -310,19 +310,17 @@ _data_manager = MetaAdsDataManager()
 @mcp_server.tool()
 @meta_api_tool
 async def search(
-    access_token: str = None,
-    query: str = None
+    query: str,
+    access_token: Optional[str] = None
 ) -> str:
     """
     Search through Meta Ads data and return matching record IDs.
-    
-    This tool is required for OpenAI ChatGPT Deep Research integration.
     It searches across ad accounts, campaigns, ads, pages, and businesses to find relevant records
     based on the provided query.
     
     Args:
-        access_token: Meta API access token (optional - will use cached token if not provided)
         query: Search query string to find relevant Meta Ads records
+        access_token: Meta API access token (optional - will use cached token if not provided)
         
     Returns:
         JSON response with list of matching record IDs
@@ -367,12 +365,10 @@ async def search(
 
 @mcp_server.tool()
 async def fetch(
-    id: str = None
+    id: str
 ) -> str:
     """
     Fetch complete record data by ID.
-    
-    This tool is required for OpenAI ChatGPT Deep Research integration.
     It retrieves the full data for a specific record identified by its ID.
     
     Args:
